// $Id: TCList.h,v 1.47 1998/05/22 11:09:59 cvs Exp $
#ifndef GTK_TCLIST_H
#define GTK_TCLIST_H
#include <gtk--/clist.h>
#include <vector>
#include <string>
#include <iterator>
#include <iostream>
#include <cassert>
#include <Picture.hh>

class TCListRow;
class TCList;

/* 2do this could be a random access iterator */
class TCListRow_iterator:bidirectional_iterator<TCListRow,ptrdiff_t>
{protected:
	typedef TCListRow T;
	vector <T *> &current_vector;
	unsigned int current_index;
	TCList *root;
	TCListRow *row; // this is the parent (if available)
	
 public:
	typedef TCListRow_iterator iterator;
 	typedef reverse_iterator<iterator> reverse_iterator;
 	TCListRow_iterator(vector<T*> &v,int i,TCList *rt)
 		: current_vector(v), current_index(i),
 		  root(rt), row(0)
 		{}
 	TCListRow_iterator(vector<T*> &v,int i,TCList *rt,TCListRow *rw)
 		: current_vector(v), current_index(i),
 		  root(rt), row(rw)
 		{}
 	TCListRow_iterator(const iterator &i)
 		: current_vector(i.current_vector),
 		  current_index(i.current_index),
 		  root(i.root), row(i.row) 
 		{}
 	
	iterator operator=(const iterator &i) // existiert nicht !!!
 	{  cerr << "TCList::iterator::operator=(const iterator &) aufgerufen\n";
 	   abort();
 	}
 	T &operator*() const 
 	{  
#ifdef BE_CAREFUL
	   assert(current_index<current_vector.size());
#endif 	
 	   return *(current_vector[current_index]);
 	}
 	iterator operator++(int)
 	{  iterator tmp=*this;
 	   current_index++;
 	   return tmp;
 	}
 	iterator &operator++()
 	{  current_index++;
 	   return *this;
 	}
 	iterator operator--(int)
 	{  iterator tmp=*this;
#ifdef BE_CAREFUL
	   assert(current_index>0);
#endif 	   
 	   current_index--;
 	   return tmp;
 	}
 	iterator &operator--()
 	{  
#ifdef BE_CAREFUL
	   assert(current_index>0);
#endif
	   current_index--;
 	   return *this;
 	}
 	bool operator==(const iterator &i) const 
 	{  return i.current_vector==current_vector 
 		&& i.current_index==current_index;
 	}
 	operator vector<T*>::iterator()
 	{  return current_vector.begin()+current_index;
 	}
	//: remove the corresponding pointer from the array
	void erase()
	{  
#ifdef BE_CAREFUL
//a	   assert(current_index>=0);
	   assert(current_index<current_vector.size());
#endif 	
	   current_vector.erase(vector<T*>::iterator(*this));
	}
	
 	
 	friend ostream &operator<<(ostream &o,const TCListRow_iterator &it);
 	friend class TCList;
 	friend class TCListRow;
};



class TCList : public Gtk_CList
{  public:
	Signal1<gpointer> delete_user;

        typedef TCListRow_iterator iterator;
        typedef TCListRow_iterator::reverse_iterator reverse_iterator;
private:
        vector <TCListRow *> children;
        gint number_of_columns;
        bool detached;
	static Picture _defaultminus,_defaultplus;
	const Picture *minus,*plus;
        
        void select_row_imp(gint,gint,GdkEventButton*);
        
        friend class TCListRow;
        friend iterator;

public:
	// construct a new TCList with titles
	TCList(const vector <string> title);
	// construct a new TCList with titles
	TCList(gint columns,const gchar **title=0);
	
	// dtor
	~TCList()
	{ 
#ifdef DEBUG
 	   cout << "TCList dtor called\n";
#endif
	   erase(begin(),end());
	}

	// return iterator to first child
        iterator begin()
        {  return iterator(children,0,this); }
        // return past the end iterator
        iterator end()
        {  return iterator(children,children.size(),this); }
        
        // actually does the same as Gtk_CList::show
	void show();

	// return (maximum) number of columns on this CList	
	gint get_number_of_columns()
	{  return number_of_columns; }
	
	// Knnte ein statisches Member werden ...
	// erase a line/subtree
	void erase(iterator position);
	// erase a range of lines/subtrees
	void erase(iterator first, iterator last);
	
	// detach from clist (erases clist)
	void detach_from_clist();
	// attach to clist (clist should be empty)
	void attach_to_clist();

        // set and get Pictures
	const Picture *getPlus();
	const Picture *getMinus();
	void setPlus(const Picture *p)
	{  plus=p; }
	void setMinus(const Picture *p)
	{  minus=p; }
};

ostream &operator<<(ostream &o,const vector<TCListRow*> &v);

#ifndef GTK_TCLISTROW_H
#include "TCListRow.h"
#endif

#endif
