// $Id: TCListRow.cc,v 1.26 1998/05/22 09:20:46 cvs Exp $
#include "TCList.h"
#include <iterator>

void TCListRow::set_expanding_column(gint col)
{  int lineno=-1;
   if (!root.detached)
   {  lineno=get_lineno();
      if (expanding_column!=-1 && lineno!=-1)
      {  gint col=expanding_column;
         expanding_column=-1;
         redraw_column_imp(lineno,col);
      }
   }
   expanding_column=col;
   if (!root.detached && lineno!=-1) redraw_column_imp(lineno,expanding_column);
}

void TCListRow::expansion_redraw_imp(gint lineno)
{  // cerr << "expansion_redraw_imp() called\n";
   if (root.detached) return;
   if (lineno==unspecified_lineno) lineno=get_lineno();
   redraw_column_imp(lineno,expanding_column);
}

void TCListRow::redraw_column_imp(gint lineno,gint colno)
{  if (lineno==-1||colno==-1||root.detached) return;
   if (colno==expanding_column)
   {  const spacing=8;
      const Picture *p= expanded ? root.getMinus() : root.getPlus();
      root.set_pixtext(lineno,expanding_column,columns[expanding_column].c_str(),
                       spacing,(GdkPixmap*)p->map,(GdkPixmap*)p->mask);
   }
   else if ((unsigned int)colno>=pics.size() || !pics[colno]) 
   {  root.set_text(lineno,colno,columns[colno].c_str());
   }  
   else
   {  const spacing=8;
      root.set_pixtext(lineno,colno,columns[colno].c_str(),
      		    spacing,(GdkPixmap*)pics[colno]->map,(GdkPixmap*)pics[colno]->mask);
   }
}

void TCListRow::redraw_pics_imp(gint lineno)
{  if (lineno==unspecified_lineno) lineno=get_lineno();
   if (lineno==-1||root.detached) return;
   if (expanding_column) redraw_column_imp(lineno,expanding_column);
   for (guint i=0;i<pics.size();i++)
   {  if ((int)i==expanding_column || !pics[i]) continue;
      redraw_column_imp(lineno,i);
   }
}

#if 1 // remove these lines by using iterator
TCListRow::TCListRow(TCListRow &parent,const vector < string > cols, gint exp_column, bool expand)
	: root(parent.root), expanded(expand), expanding_column(exp_column),
	  columns(cols), user_data(0)
{  if (!root.detached && parent.expanded)
   {  int newline;

      if (!parent.children.empty()) 
      {  newline=parent.children.back()->get_lineno();
         if (newline!=-1) newline+=parent.children.back()->visible_size();
      }
      else 
      {  newline=parent.get_lineno();
         if (newline!=-1) newline++;
      }
      
      parent.children.push_back(this);
      
      if (newline != -1)
      {	 const gchar *arr[cols.size()];
	 for (guint i = 0; i < cols.size(); i++) arr[i] = cols[i].c_str();
	 root.insert_row(newline, (gchar**)arr);
	 root.set_row_data(newline, this);
	 redraw_pics_imp(newline);
      }
   }
   else parent.children.push_back(this);
}

TCListRow::TCListRow(TCListRow &parent,const gchar **arr,gint exp_column,bool expand)
	: root(parent.root), expanded(expand), expanding_column(exp_column),
	  user_data(0)
{  for (int i=0;i<root.get_number_of_columns();i++) 
      columns.push_back(string(arr[i]));
   if (!root.detached && parent.expanded) 
   {  int newline;
      if (!parent.children.empty()) 
      {  newline=parent.children.back()->get_lineno();
         if (newline!=-1) newline+=parent.children.back()->visible_size();
      }
      else 
      {  newline=parent.get_lineno();
         if (newline!=-1) newline++;
      }
      parent.children.push_back(this);

      if (newline!=-1)
      {  root.insert_row(newline,(gchar**)arr);
         root.set_row_data(newline,this);
         redraw_pics_imp(newline);
      }
   }
   else parent.children.push_back(this);
}

TCListRow::TCListRow(TCList &parent,const vector < string > cols, gint exp_column, bool expand)
	: root(parent), expanded(expand), expanding_column(exp_column),
	  columns(cols), user_data(0)
{  parent.children.push_back(this);

   if (!root.detached)
   {  const gchar *arr[cols.size()];
      for (guint i = 0; i < cols.size(); i++) arr[i] = cols[i].c_str();
   
      gint newline=root.append((gchar**)arr);
      root.set_row_data(newline, this);
      redraw_pics_imp(newline);
   }
}

TCListRow::TCListRow(TCList &parent,const gchar **arr,gint exp_column,bool expand)
	: root(parent), expanded(expand), expanding_column(exp_column),
	  user_data(0)
{  for (int i=0;i<root.get_number_of_columns();i++) 
      columns.push_back(string(arr[i]));
   parent.children.push_back(this);
   
   if (!root.detached)
   {  gint newline=root.append((gchar**)arr);
      root.set_row_data(newline,this);
      redraw_pics_imp(newline);
   }
}
#endif

void TCListRow::expand()
{  if (expanded) return;
   expanded=true;
   if (root.detached) return;
   root.freeze();
   expand_imp();
   root.thaw();
#ifdef DEBUG   
   cout << "expand " << *this;
#endif   
}

void TCListRow::expand_imp()
{  if (root.detached) return;
   int lineno=get_lineno();
   if (lineno==-1) return;
   redraw_column_imp(lineno,expanding_column);
   lineno++;
   // expand direct children
   for (iterator i=begin();i!=end();i++)
   {  const vector <string> &v=(*i).get_columns();
      const gchar *arr[v.size()];

      for (guint j=0;j<v.size();j++) arr[j]=v[j].c_str();
      root.insert_row(lineno,(gchar**)arr);
      root.set_row_data(lineno,&*i);
      (*i).redraw_pics_imp(lineno);
      lineno++;
   }
   // now expand grandchildren
   for (iterator i=begin();i!=end();i++)
   {  if ((*i).expanded) (*i).expand_imp();
   }
}

void TCListRow::collapse()
{  if (!expanded) return;
   expanded=false;
   if (root.detached) return;
   root.freeze();
   collapse_imp();
   root.thaw();
}

void TCListRow::collapse_imp()
{  if (root.detached) return;
   for (iterator i=begin();i!=end();i++)
   {  (*i).remove_imp();
   }
   redraw_column_imp(get_lineno(),expanding_column);
}

void TCListRow::remove_imp()
{  if (root.detached) return;
   int lineno=get_lineno();

#ifdef DEBUG
   cout << "remove:\n" << *this;
#endif   
   if (lineno!=-1) 
   {  root.remove_row(lineno);
      // remove children
      if (expanded) for (iterator i=begin();i!=end();i++)
      {  (*i).remove_imp();
      }
   }
}

ostream &operator<<(ostream &out,TCListRow &lr)
{    out << "TCListRow @" << &lr << " " << (lr.expanded?"":"!") << "exp";
     out << " exp_col=" << lr.expanding_column << " " << lr.children.size();
     out << " children: ";
     for (int i=0;i<lr.root.get_number_of_columns();i++)
     {  out << "\"" << lr.columns[i] << "\" ";
     }
     out << "\n";
#ifdef DEBUG
     for (TCListRow::iterator i=lr.begin();i!=lr.end();i++)
        out << "\t" << *i;
#endif     
     return out;
} 

void TCListRow::toggle_expansion()
{  if (expanded) collapse();
   else expand();
}

void TCListRow::reparent_children(TCListRow &newparent)
{  if (!root.detached) collapse_imp();
   if (!root.detached && newparent.expanded) newparent.collapse_imp();
   back_insert_iterator<vector<TCListRow*> > ii(newparent.children);
   copy(children.begin(),children.end(),ii);
   children.erase(children.begin(),children.end());
   if (!root.detached && newparent.expanded) newparent.expand_imp();
}

int TCListRow::visible_size() const
{  if (root.detached) return 0;
   int res=1;
   if (expanded)
   {  for (iterator i=begin();i!=end();i++)
         res+=(*i).expanded ? (*i).visible_size() : 1;
         // this '?:' avoids one depth of recursion
   }
   return res;
}

void TCListRow::relabel(int column,string text)
{  // if (column>root.get_number_of_columns()) return;
   columns[column]=text;
   if (root.detached) return;
   gint lineno=get_lineno();
   if (lineno==-1) return;
   redraw_column_imp(lineno,column);
}

TCListRow::TCListRow(iterator &next,const vector < string > cols, gint exp_column, bool expand)
	: root(*(next.root)), expanded(expand), expanding_column(exp_column),
	  columns(cols), user_data(0)
{  vector < TCListRow * >::iterator viter = next;
   bool is_end = next.current_index >= next.current_vector.size();

   next.current_vector.insert(viter, this);

   if (!root.detached)
   {  gint newline;
      const gchar *arr[cols.size()];

      for (guint i = 0; i < cols.size(); i++) arr[i] = cols[i].c_str();

      if (!is_end)
      {	 newline = (*next).get_lineno();
	 if (newline != -1) root.insert_row(newline, (gchar **) arr);
	 else
	 {  // when next node is not visible, we shouldn't be visible either ?
	 }
      }
      else
      {	 if (next.row)
	 {  if (next.row->empty())
	    {  newline=next.row->get_lineno();
	       if (newline!=-1) newline++;
	    }
	    else
	    {  newline=next.row->children.back()->get_lineno();
	       if (newline!=-1) newline+=next.row->children.back()->visible_size();
	    }
	    if (newline!=-1) root.insert_row(newline, (gchar**)arr);
	 }
	 else
	 {  newline = root.append((gchar **) arr);
	 }
      }

      if (newline != -1)
      {	 root.set_row_data(newline, this);
	 redraw_pics_imp(newline);
      }
   }
}

void TCListRow::append_imp()
{  if (root.detached) return;
   
   const gchar *arr[columns.size()];
   
   for (guint j=0;j<columns.size();j++) arr[j]=columns[j].c_str();
   gint lineno=root.append((gchar**)arr);
   root.set_row_data(lineno,this);
   redraw_pics_imp(lineno);

   if (expanded) for (iterator i=begin();i!=end();++i)
   {  (*i).append_imp();
   }
}
