// $Id: TCListRow.h,v 1.18 1998/05/22 09:12:47 cvs Exp $
#ifndef GTK_TCLISTROW_H
#define GTK_TCLISTROW_H
#include <vector>
#include <string>
#include <iterator>
#include <iostream>
#include <algo.h>

#ifdef DEBUG
#include "misc.hh"
#endif

class TCListRow;

#ifndef GTK_TCLIST_H
#  include "TCList.h"
#endif

class TCListRow
{public:
	// this is an STL like iterator for TCListRows
	typedef TCList::iterator iterator;
   
private:
   friend class TCList;
   friend iterator;
   
   TCList &root;
   vector <TCListRow *> children;
   bool expanded;
   gint expanding_column;
   vector <string> columns;
   vector <const Picture*> pics;
   gpointer user_data;
   const unspecified_lineno=-2;
   
   // ??? is called by parent->show() !!!
   void expand_imp(); 
   void collapse_imp();
   void redraw_column_imp(gint lineno,gint colno);
   void redraw_pics_imp(gint lineno=unspecified_lineno);
   //: do not use too extensively, use redraw_column_imp where possible
   void expansion_redraw_imp(gint lineno=unspecified_lineno);
   void remove_imp();
   // for a very fast insertion of the whole list
   void append_imp(); 
   void select_row_imp(gint,gint,GdkEventButton*);
   
   virtual void delete_imp()
   {  
#ifdef DEBUG   
      cout << "delete_imp @" << this << "\n";
#endif      
      if (!children.empty()) root.erase(begin(),end());
      root.delete_user(user_data);
#ifdef DEBUG      
      cout << "!delete_imp@" << this << "\n";
#endif      
   }
public:
	
	//: this was named addLine ... better as a constructor (inheritance)
   // add a line. If it contains a subtree please specify the expanding column
   // <br>there is also a gchar**-version of this function
	TCListRow(TCListRow &parent,const vector <string>,gint exp_column=-1,bool expand=false);
	//: this was named addLine ... better as a constructor (inheritance)
	TCListRow(TCListRow &parent,const gchar **contents,gint exp_column=-1,bool expand=false);
	//: this was named addLine ... better as a constructor (inheritance)
   // add a line. If it contains a subtree please specify the expanding column
   // <br>there is also a gchar**-version of this function
	TCListRow(TCList &parent,const vector <string>,gint exp_column=-1,bool expand=false);
	//: this was named addLine ... better as a constructor (inheritance)
	TCListRow(TCList &parent,const gchar **contents,gint exp_column=-1,bool expand=false);
   // insert a line before the iterator. If it contains a subtree please specify the expanding column
	TCListRow(iterator &next,const vector <string>,gint exp_column=-1,bool expand=false);
	
	virtual ~TCListRow() 
	{  delete_imp(); 
#ifdef DEBUG	
	   write_hex(cout,this) << " dtor end\n";
#endif	   
	}
	//: return iterator to first child
        iterator begin() 
        {  return TCList::iterator(children,0,&root,this); }
        //: return past the end iterator
        iterator end() 
        {  return TCList::iterator(children,children.size(),&root,this); }
        //: whether this Node has no children
        bool empty() const 
        {  return children.empty(); }
        //: number of children
        int size() const
        {  return children.size(); }

   //: specify the expanding column (the one with the (+)-sign)
   void set_expanding_column(gint col);
   //: return the expanding column
   gint get_expanding_column()
   {  return expanding_column; }
   //: return line number within Gtk_CList (or -1 if not expanded)
   gint get_lineno() const
   {  return root.find_row_from_data((gpointer)this); }
   //: not implemented. should return number of lines of this subtree
   gint get_num_rows() const;
   //: add an hook for your data
   void set_user_data(gpointer x)
   {  user_data=x;
   }
   //: return your data
   gpointer get_user_data() const
   {  return user_data;
   }
   //: expand this subtree
   void expand();
   //: collapse this subtree
   void collapse();
   //: flip expansion of this subtree
   void toggle_expansion();

   //: return contents (text) of this node
   vector <string> &get_columns()
   {  return columns; }
   
   //: reparent children to parent
   void reparent_children(TCListRow &newparent);
   
   //: find iterator to child
   iterator find(const TCListRow &value)
   {  return ::find(begin(),end(),value);
   }
   //: visible_size of this subtree
   int visible_size() const;
   
   //: relabel column
   void relabel(int column, string text);
   void relabel(int column, gchar *text)
   {  relabel(column,string(text)); }
   
   bool operator==(const TCListRow &value) const { return this==&value; }
   
   // For performance reasons cloning Pictures is not done.
   // You really need a picture _ONCE_? You're kidding!
   void setPicture(int column,const Picture *pic)
   {  while (pics.size()<=(unsigned int)column) pics.push_back(0);
      pics[column]=pic;
      if (!root.detached) redraw_column_imp(get_lineno(),column);
   }
   
   friend ostream &operator<<(ostream &out,TCListRow &lr);
};

//: debug contents of this node
ostream &operator<<(ostream &out,TCListRow &lr);

#endif
